/*
 * Copyright (c) 2012, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * */

/*
 *  ======== EventHdr2.h ========
 * This file is included to provide users of previous releases
 * of cUIA and users of the UIA target content package with 
 * the macros necessary to parse UIAPacket packets and
 * extract snapshot event data from the packets.
 * These macros have been added to the ti/uia/runtime/EventHdr.h
 * file that ships with this release, so users should not
 * use this file in projects being built with cuia_1_00_00_06_eng or later.
 */

#ifndef ti_uia_runtime_EventHdr2__include
#define ti_uia_runtime_EventHdr2__include

#ifndef __nested__
#define __nested__
#define ti_uia_runtime_EventHdr__top__
#endif

#ifdef __cplusplus
#define __extern extern "C"
#else
#define __extern extern
#endif

#define ti_uia_runtime_EventHdr2___VERS 100


/*
 * ======== INCLUDES ========
 */

#include <xdc/std.h>

#include <xdc/runtime/xdc.h>
//#include <xdc/runtime/Types.h>
#include <ti/uia/runtime/EventHdr.h>

#ifndef EventHdr_SNAPSHOTDATA_TOTALLEN_BIT_SHIFT

#define EventHdr_SNAPSHOTDATA_TOTALLEN_BIT_SHIFT 0
#define EventHdr_SNAPSHOTDATA_TOTALLEN_MASK 0xFFFFL
#define EventHdr_SNAPSHOTDATA_TOTALLEN_SHIFTED_MASK 0xFFFF

#define EventHdr_SNAPSHOTDATA_RECLEN_BIT_SHIFT 16
#define EventHdr_SNAPSHOTDATA_RECLEN_MASK 0xFFFFL
#define EventHdr_SNAPSHOTDATA_RECLEN_SHIFTED_MASK 0xFFFF0000L

typedef struct ti_uia_runtime_EventHdr_Hdr {
    xdc_Bits32 word0;
    xdc_Bits32 word1;
    xdc_Bits32 word2;
    xdc_Bits32 word3;
    xdc_Bits32 word4;
    xdc_Bits32 word5;
    xdc_Bits32 word6;
    xdc_Bits32 word7;
    xdc_Bits32 word8;
    xdc_Bits32 word9;
    xdc_Bits32 word10;
    xdc_Bits32 word11;
    xdc_Bits32 word12;
} ti_uia_runtime_EventHdr_Hdr;

/* HdrType 2: EventWithSnapshotId
  *    word0: EventHdr
 *    word1: event Id (top 16 bits) & module Id (bottom 16 bits)
 *    word2: filename pointer
 *    word3: linenum
 *    word4: snapshotId
 *    word5: address where the data was located
 *    word6: total length of data (top 16-bits)
 *           length for this record (bottom 16 bits)
 *    word7: format pointer
 *    data:  the rest of the record contains the data
 */
#define EventHdr_HDRTYPE2_SNAPSHOTID_WORD word4
#define EventHdr_HDRTYPE2_SNAPSHOTDATAADRS_WORD word5
#define EventHdr_HDRTYPE2_SNAPSHOTTOTALLEN_WORD word6
#define EventHdr_HDRTYPE2_SNAPSHOTRECLEN_WORD word6
#define EventHdr_HDRTYPE2_SNAPSHOTFMT_WORD word7
#define EventHdr_HDRTYPE2_SNAPSHOTPAYLOAD_WORD word8

 /* HdrType 3: EventWithSnapshotIdAndTimestamp
 *    word0: EventHdr
 *    word1: Timestamp lower 32 bits
 *    word2: Timestamp upper 32 bits
 *    word3: event Id (top 16 bits) & module Id (bottom 16 bits)
 *    word4: filename pointer
 *    word5: linenum
 *    word6: snapshotId
 *    word7: address where the data was located
 *    word8: total length of data (top 16-bits)
 *           length for this record (bottom 16 bits)
 *    word9: format pointer
 *    data:  the rest of the record contains the data
 */

#define EventHdr_HDRTYPE3_SNAPSHOTID_WORD word6
#define EventHdr_HDRTYPE3_SNAPSHOTDATAADRS_WORD word7
#define EventHdr_HDRTYPE3_SNAPSHOTTOTALLEN_WORD word8
#define EventHdr_HDRTYPE3_SNAPSHOTRECLEN_WORD word8
#define EventHdr_HDRTYPE3_SNAPSHOTFMT_WORD word9
#define EventHdr_HDRTYPE3_SNAPSHOTPAYLOAD_WORD word10

/*
 *  HdrType 7: EventWithSnapshotIdAndTimestampAndEndpointId:
 *    word0: EventHdr
 *    word1: Timestamp lower 32 bits
 *    word2: Timestamp upper 32 bits
 *    word3: EndpointId (lower 16 bits)
 *    word4: ProcessId
 *    word5: event Id (top 16 bits) & module Id (bottom 16 bits)
 *    word6: filename pointer
 *    word7: linenum
 *    word8: snapshotId
 *    word9: address where the data was located
 *    word10: total length of data (top 16-bits)
 *           length for this record (bottom 16 bits)
 *    word11: format pointer
 *    data:  the rest of the record contains the data
 */
#define EventHdr_HDRTYPE7_SNAPSHOTID_WORD word8
#define EventHdr_HDRTYPE7_SNAPSHOTDATAADRS_WORD word9
#define EventHdr_HDRTYPE7_SNAPSHOTTOTALLEN_WORD word10
#define EventHdr_HDRTYPE7_SNAPSHOTRECLEN_WORD word10
#define EventHdr_HDRTYPE7_SNAPSHOTFMT_WORD word11
#define EventHdr_HDRTYPE7_SNAPSHOTPAYLOAD_WORD word12

 /* The following definition is to be merged in from UIAcTools:
  * ti_uia_runtime_EventHdr_HdrType_EventWithSnapshotIdAndTimestampAndEndpointId = 7,
  */
#ifndef ti_uia_runtime_EventHdr_HdrType_EventWithSnapshotIdAndTimestampAndEndpointId
#define ti_uia_runtime_EventHdr_HdrType_EventWithSnapshotIdAndTimestampAndEndpointId (7)
#endif

/*
 *  ======== EventHdr_isASnapshotEvent ========
 *  returns 1 (TRUE) if the event is a snapshot event, else returns 0 (FALSE).
 */
#define  ti_uia_runtime_EventHdr_isASnapshotEvent(pEventHdr)   \
   ((ti_uia_runtime_EventHdr_getHdrType(*(UInt32 *)pEventHdr) == ti_uia_runtime_EventHdr_HdrType_EventWithSnapshotId) ? \
        1 :  \
		((ti_uia_runtime_EventHdr_getHdrType(*(UInt32 *)pEventHdr) == ti_uia_runtime_EventHdr_HdrType_EventWithSnapshotIdAndTimestamp) ? \
		1 : \
		((ti_uia_runtime_EventHdr_getHdrType(*(UInt32 *)pEventHdr) == ti_uia_runtime_EventHdr_HdrType_EventWithSnapshotIdAndTimestampAndEndpointId) ? \
		1 : 0)) \
    )
/*
 *  ======== EventHdr_getSnapshotId ========
 *  returns the snapshot ID.
 *  If snapshot data length > LogSnapshot_maxEventSize,
 *  multiple events will be used to transmit the snapshot data,
 *  all of which will have the same snapshot ID value.
 *  Returns -1 if the event is not a snapshot event.
 */
#define  ti_uia_runtime_EventHdr_getSnapshotId(pEventHdr)   \
		((ti_uia_runtime_EventHdr_getHdrType(*(UInt32 *)pEventHdr) == ti_uia_runtime_EventHdr_HdrType_EventWithSnapshotId) ? \
		((ti_uia_runtime_EventHdr_Hdr *)pEventHdr)->EventHdr_HDRTYPE2_SNAPSHOTID_WORD :  \
		((ti_uia_runtime_EventHdr_getHdrType(*(UInt32 *)pEventHdr) == ti_uia_runtime_EventHdr_HdrType_EventWithSnapshotIdAndTimestamp) ? \
		((ti_uia_runtime_EventHdr_Hdr *)pEventHdr)->EventHdr_HDRTYPE3_SNAPSHOTID_WORD : \
		((ti_uia_runtime_EventHdr_getHdrType(*(UInt32 *)pEventHdr) == ti_uia_runtime_EventHdr_HdrType_EventWithSnapshotIdAndTimestampAndEndpointId) ? \
		((ti_uia_runtime_EventHdr_Hdr *)pEventHdr)->EventHdr_HDRTYPE7_SNAPSHOTID_WORD : -1)) \
      )

/*
 *  ======== EventHdr_getSnapshotData ========
 *  returns a pointer to the snapshot data carried by the event.
 *  Returns null if the event is not a snapshot event.
 */
#define  ti_uia_runtime_EventHdr_getSnapshotData(pEventHdr)   \
   (void *)((ti_uia_runtime_EventHdr_getHdrType(*(UInt32 *)pEventHdr) == ti_uia_runtime_EventHdr_HdrType_EventWithSnapshotId) ? \
        &((ti_uia_runtime_EventHdr_Hdr *)pEventHdr)->EventHdr_HDRTYPE2_SNAPSHOTPAYLOAD_WORD :  \
		((ti_uia_runtime_EventHdr_getHdrType(*(UInt32 *)pEventHdr) == ti_uia_runtime_EventHdr_HdrType_EventWithSnapshotIdAndTimestamp) ? \
		&((ti_uia_runtime_EventHdr_Hdr *)pEventHdr)->EventHdr_HDRTYPE3_SNAPSHOTPAYLOAD_WORD : \
		((ti_uia_runtime_EventHdr_getHdrType(*(UInt32 *)pEventHdr) == ti_uia_runtime_EventHdr_HdrType_EventWithSnapshotIdAndTimestampAndEndpointId) ? \
		&((ti_uia_runtime_EventHdr_Hdr *)pEventHdr)->EventHdr_HDRTYPE7_SNAPSHOTPAYLOAD_WORD : NULL)) \
    )

/*
 * ======== EventHdr_getSnapshotFmtStringAdrs ========
 * returns the address the string constant used as a format string in the snapshot event.
 * Returns 0 if the event is not a snapshot event.
 */
#define ti_uia_runtime_EventHdr_getSnapshotFmtStringAdrs(pEventHdr)  \
   ((ti_uia_runtime_EventHdr_getHdrType(*(UInt32 *)pEventHdr) == ti_uia_runtime_EventHdr_HdrType_EventWithSnapshotId) ? \
		((ti_uia_runtime_EventHdr_Hdr *)pEventHdr)->EventHdr_HDRTYPE2_SNAPSHOTFMT_WORD :  \
		((ti_uia_runtime_EventHdr_getHdrType(*(UInt32 *)pEventHdr) == ti_uia_runtime_EventHdr_HdrType_EventWithSnapshotIdAndTimestamp) ? \
		((ti_uia_runtime_EventHdr_Hdr *)pEventHdr)->EventHdr_HDRTYPE3_SNAPSHOTFMT_WORD : \
		((ti_uia_runtime_EventHdr_getHdrType(*(UInt32 *)pEventHdr) == ti_uia_runtime_EventHdr_HdrType_EventWithSnapshotIdAndTimestampAndEndpointId) ? \
		((ti_uia_runtime_EventHdr_Hdr *)pEventHdr)->EventHdr_HDRTYPE7_SNAPSHOTFMT_WORD : 0)) \
    )

/*
 * ======== EventHdr_getSnapshotDataAdrs ========
 * returns the address the snapshot data was read from.
 * Returns 0 if the event is not a snapshot event.
 */
#define ti_uia_runtime_EventHdr_getSnapshotDataAdrs(pEventHdr)  \
   ((ti_uia_runtime_EventHdr_getHdrType(*(UInt32 *)pEventHdr) == ti_uia_runtime_EventHdr_HdrType_EventWithSnapshotId) ? \
		((ti_uia_runtime_EventHdr_Hdr *)pEventHdr)->EventHdr_HDRTYPE2_SNAPSHOTDATAADRS_WORD :  \
		((ti_uia_runtime_EventHdr_getHdrType(*(UInt32 *)pEventHdr) == ti_uia_runtime_EventHdr_HdrType_EventWithSnapshotIdAndTimestamp) ? \
		((ti_uia_runtime_EventHdr_Hdr *)pEventHdr)->EventHdr_HDRTYPE3_SNAPSHOTDATAADRS_WORD : \
		((ti_uia_runtime_EventHdr_getHdrType(*(UInt32 *)pEventHdr) == ti_uia_runtime_EventHdr_HdrType_EventWithSnapshotIdAndTimestampAndEndpointId) ? \
		((ti_uia_runtime_EventHdr_Hdr *)pEventHdr)->EventHdr_HDRTYPE7_SNAPSHOTDATAADRS_WORD : 0)) \
    )

/*
 * ======== EventHdr_getSnapshotTotalLength ========
 * returns the total length of the entire original snapshot data.
 * Note that this may be less than the sum of the payloads carried by all
 * of the events used to transport the data if the original snapshot length
 * was not divisible by 4 (number of bytes per word).
 */
#define ti_uia_runtime_EventHdr_getSnapshotTotalLength(pEventHdr) \
   ((ti_uia_runtime_EventHdr_getHdrType(*(UInt32 *)pEventHdr) == ti_uia_runtime_EventHdr_HdrType_EventWithSnapshotId) ? \
        ((((ti_uia_runtime_EventHdr_Hdr *)pEventHdr)->EventHdr_HDRTYPE2_SNAPSHOTTOTALLEN_WORD >> EventHdr_SNAPSHOTDATA_TOTALLEN_BIT_SHIFT) & EventHdr_SNAPSHOTDATA_TOTALLEN_MASK) :  \
		((ti_uia_runtime_EventHdr_getHdrType(*(UInt32 *)pEventHdr) == ti_uia_runtime_EventHdr_HdrType_EventWithSnapshotIdAndTimestamp) ? \
		((((ti_uia_runtime_EventHdr_Hdr *)pEventHdr)->EventHdr_HDRTYPE3_SNAPSHOTTOTALLEN_WORD >> EventHdr_SNAPSHOTDATA_TOTALLEN_BIT_SHIFT) & EventHdr_SNAPSHOTDATA_TOTALLEN_MASK) : \
		((ti_uia_runtime_EventHdr_getHdrType(*(UInt32 *)pEventHdr) == ti_uia_runtime_EventHdr_HdrType_EventWithSnapshotIdAndTimestampAndEndpointId) ? \
		((((ti_uia_runtime_EventHdr_Hdr *)pEventHdr)->EventHdr_HDRTYPE7_SNAPSHOTTOTALLEN_WORD >> EventHdr_SNAPSHOTDATA_TOTALLEN_BIT_SHIFT) & EventHdr_SNAPSHOTDATA_TOTALLEN_MASK) : 0)) \
    )

/*
 * ======== EventHdr_getSnapshotEventDataLength ========
 * returns the length of the portion of the snapshot data sent in the event .
 */
#define ti_uia_runtime_EventHdr_getSnapshotEventDataLength(pEventHdr) \
   ((ti_uia_runtime_EventHdr_getHdrType(*(UInt32 *)pEventHdr) == ti_uia_runtime_EventHdr_HdrType_EventWithSnapshotId) ? \
        ((((ti_uia_runtime_EventHdr_Hdr *)pEventHdr)->EventHdr_HDRTYPE2_SNAPSHOTRECLEN_WORD >> EventHdr_SNAPSHOTDATA_RECLEN_BIT_SHIFT) & EventHdr_SNAPSHOTDATA_RECLEN_MASK) :  \
		((ti_uia_runtime_EventHdr_getHdrType(*(UInt32 *)pEventHdr) == ti_uia_runtime_EventHdr_HdrType_EventWithSnapshotIdAndTimestamp) ? \
		((((ti_uia_runtime_EventHdr_Hdr *)pEventHdr)->EventHdr_HDRTYPE3_SNAPSHOTRECLEN_WORD >> EventHdr_SNAPSHOTDATA_RECLEN_BIT_SHIFT) & EventHdr_SNAPSHOTDATA_RECLEN_MASK) : \
		((ti_uia_runtime_EventHdr_getHdrType(*(UInt32 *)pEventHdr) == ti_uia_runtime_EventHdr_HdrType_EventWithSnapshotIdAndTimestampAndEndpointId) ? \
		((((ti_uia_runtime_EventHdr_Hdr *)pEventHdr)->EventHdr_HDRTYPE7_SNAPSHOTRECLEN_WORD >> EventHdr_SNAPSHOTDATA_RECLEN_BIT_SHIFT) & EventHdr_SNAPSHOTDATA_RECLEN_MASK) : 0)) \
    )

#define ti_uia_runtime_EventHdr_sizeOfEventHeader_EventWithSnapshotIdAndTimestamp (offsetof(ti_uia_runtime_EventHdr_Hdr,EventHdr_HDRTYPE3_SNAPSHOTPAYLOAD_WORD))

/*
 * ======== PREFIX ALIASES ========
 */

/* module prefix */
#define EventHdr_Hdr ti_uia_runtime_EventHdr_Hdr
#define EventHdr_isASnapshotEvent ti_uia_runtime_EventHdr_isASnapshotEvent
#define EventHdr_getSnapshotId ti_uia_runtime_EventHdr_getSnapshotId
#define EventHdr_getSnapshotData ti_uia_runtime_EventHdr_getSnapshotData
#define EventHdr_getSnapshotFmtStringAdrs ti_uia_runtime_EventHdr_getSnapshotFmtStringAdrs
#define EventHdr_getSnapshotDataAdrs ti_uia_runtime_EventHdr_getSnapshotDataAdrs
#define EventHdr_getSnapshotEventDataLength ti_uia_runtime_EventHdr_getSnapshotEventDataLength
#define EventHdr_getSnapshotTotalLength ti_uia_runtime_EventHdr_getSnapshotTotalLength
#define EventHdr_sizeOfEventHeader_EventWithSnapshotIdAndTimestamp ti_uia_runtime_EventHdr_sizeOfEventHeader_EventWithSnapshotIdAndTimestamp

#endif

#endif
