/*
 * Copyright (c) 2012, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * */

/*
 * ======== PROLOGUE ========
 */

#ifndef ti_uiaplus_loggers_stream_LoggerStreamer__include
#define ti_uiaplus_loggers_stream_LoggerStreamer__include

#ifndef __nested__
#define __nested__
#define ti_uiaplus_loggers_stream_LoggerStreamer__top__
#endif

#ifdef __cplusplus
#define __extern extern "C"
#else
#define __extern extern
#endif

#define ti_uiaplus_loggers_stream_LoggerStreamer___VERS 150


/*
 * ======== INCLUDES ========
 */

#include <xdc/std.h>
#include <xdc/runtime/xdc.h>
#include <xdc/runtime/Types.h>
#include <ti/uiaplus/loggers/stream/LoggerStreamer__prologue.h>
#include <xdc/runtime/Log.h>
#include <ti/uiaplus/loggers/stream/crc.h>

extern Void LoggerStreamer_init(UInt32* buffer1, UInt32 bufSizeInMAUs);
extern Bool LoggerStreamer_prime(Ptr buffer);
extern Void LoggerStreamer_flush();
extern Void UIALogger_setMask(String src);
extern crc LoggerStreamer_generateCRC(unsigned char const message[], int nBytes);
 
/* Init LoggerFlexSupport module */
extern Void LoggerFlexSupport_init(Void);

/* Exit LoggerFlexSupport module */
extern Void LoggerFlexSupport_exit(Void);

/* write a logger event which has no arguments */
extern Void LoggerFlexSupport_write0(xdc_runtime_Log_Event evt, xdc_runtime_Types_ModuleId mid);

/* write a logger event which has 1 argument */
extern Void LoggerFlexSupport_write1(xdc_runtime_Log_Event evt, xdc_runtime_Types_ModuleId mid, IArg a1);

/* write a logger event which has 2 arguments */
extern Void LoggerFlexSupport_write2(xdc_runtime_Log_Event evt, xdc_runtime_Types_ModuleId mid, IArg a1, 
    IArg a2);

/* write a logger event which has 4 arguments */
extern Void LoggerFlexSupport_write4(xdc_runtime_Log_Event evt, xdc_runtime_Types_ModuleId mid, IArg a1, 
    IArg a2, IArg a3, IArg a4);

/* write a logger event which has 8 arguments */
extern Void LoggerFlexSupport_write8(xdc_runtime_Log_Event evt, xdc_runtime_Types_ModuleId mid, IArg a1, 
    IArg a2, IArg a3, IArg a4, IArg a5, IArg a6, IArg a7, IArg a8);

extern Void LoggerFlexSupport_writeMemoryRange(xdc_runtime_Log_Event evt,
		xdc_runtime_Types_ModuleId mid, UInt32 snapshotId,IArg fileName, IArg lineNum, IArg fmt,
		IArg startAdrs, UInt32 lengthInMAUs);
        
/* Disable logging */
extern Bool LoggerFlexSupport_disable();

/* Enable logging */
extern Bool LoggerFlexSupport_enable();
extern volatile UInt32 UIALogger_Diags_RuntimeOn;

/*
 * ======== AUXILIARY DEFINITIONS ========
 */

/* TransportType */
enum ti_uiaplus_loggers_stream_LoggerStreamer_TransportType {
    ti_uiaplus_loggers_stream_LoggerStreamer_TransportType_UART = 0,
    ti_uiaplus_loggers_stream_LoggerStreamer_TransportType_USB = 1,
    ti_uiaplus_loggers_stream_LoggerStreamer_TransportType_UDP = 2,
    ti_uiaplus_loggers_stream_LoggerStreamer_TransportType_TCP = 3,
    ti_uiaplus_loggers_stream_LoggerStreamer_TransportType_CUSTOM = 4
};
typedef enum ti_uiaplus_loggers_stream_LoggerStreamer_TransportType ti_uiaplus_loggers_stream_LoggerStreamer_TransportType;


/*
 * ======== EXTERNAL FUNCTIONS ========
 */

/*
 * ======== INTERNAL DEFINITIONS ========
 */

/* WRITE0_SIZE_IN_BYTES */
#define ti_uiaplus_loggers_stream_LoggerStreamer_WRITE0_SIZE_IN_BYTES (8)

/* WRITE1_SIZE_IN_BYTES */
#define ti_uiaplus_loggers_stream_LoggerStreamer_WRITE1_SIZE_IN_BYTES (12)

/* WRITE2_SIZE_IN_BYTES */
#define ti_uiaplus_loggers_stream_LoggerStreamer_WRITE2_SIZE_IN_BYTES (16)

/* WRITE4_SIZE_IN_BYTES */
#define ti_uiaplus_loggers_stream_LoggerStreamer_WRITE4_SIZE_IN_BYTES (24)

/* WRITE8_SIZE_IN_BYTES */
#define ti_uiaplus_loggers_stream_LoggerStreamer_WRITE8_SIZE_IN_BYTES (40)

/* TIMESTAMP */
#define ti_uiaplus_loggers_stream_LoggerStreamer_TIMESTAMP (8)

/* NO_TIMESTAMP */
#define ti_uiaplus_loggers_stream_LoggerStreamer_NO_TIMESTAMP (0)


#define LoggerStreamer_WRITE0_SIZE_IN_BYTES ti_uiaplus_loggers_stream_LoggerStreamer_WRITE0_SIZE_IN_BYTES
#define LoggerStreamer_WRITE1_SIZE_IN_BYTES ti_uiaplus_loggers_stream_LoggerStreamer_WRITE1_SIZE_IN_BYTES
#define LoggerStreamer_WRITE2_SIZE_IN_BYTES ti_uiaplus_loggers_stream_LoggerStreamer_WRITE2_SIZE_IN_BYTES
#define LoggerStreamer_WRITE4_SIZE_IN_BYTES ti_uiaplus_loggers_stream_LoggerStreamer_WRITE4_SIZE_IN_BYTES
#define LoggerStreamer_WRITE8_SIZE_IN_BYTES ti_uiaplus_loggers_stream_LoggerStreamer_WRITE8_SIZE_IN_BYTES
#define LoggerStreamer_TIMESTAMP ti_uiaplus_loggers_stream_LoggerStreamer_TIMESTAMP
#define LoggerStreamer_NO_TIMESTAMP ti_uiaplus_loggers_stream_LoggerStreamer_NO_TIMESTAMP

/*
 * ======== MODULE-WIDE CONFIGS ========
 */


/*
 * ======== FUNCTION DECLARATIONS ========
 */
/*
 * ======== GENERATED EVENT CONSTANTS ========
 */
#define ti_uiaplus_loggers_stream_LoggerStreamer_L_test (0x1D5B0100)
#define ti_uiaplus_loggers_stream_LoggerStreamer_L_test__msg "Test"
#define LoggerStreamer_L_test (0x1D5B0100)
#define LoggerStreamer_L_test__msg "Test" 

/*
 * ======== EPILOGUE ========
 */

#include <ti/uiaplus/loggers/stream/LoggerStreamer__epilogue.h>

#ifdef ti_uiaplus_loggers_stream_LoggerStreamer__top__
#undef __nested__
#endif

#endif /* ti_uiaplus_loggers_stream_LoggerStreamer__include */
/*
 * ======== PREFIX ALIASES ========
 */

#if !defined(__nested__) && !defined(ti_uiaplus_loggers_stream_LoggerStreamer__nolocalnames)

#ifndef ti_uiaplus_loggers_stream_LoggerStreamer__localnames__done
#define ti_uiaplus_loggers_stream_LoggerStreamer__localnames__done

/* module prefix */
#define LoggerStreamer_Module ti_uiaplus_loggers_stream_LoggerStreamer_Module
#define LoggerStreamer_TransportType ti_uiaplus_loggers_stream_LoggerStreamer_TransportType
#define LoggerStreamer_TransportType_UART ti_uiaplus_loggers_stream_LoggerStreamer_TransportType_UART
#define LoggerStreamer_TransportType_USB ti_uiaplus_loggers_stream_LoggerStreamer_TransportType_USB
#define LoggerStreamer_TransportType_UDP ti_uiaplus_loggers_stream_LoggerStreamer_TransportType_UDP
#define LoggerStreamer_TransportType_TCP ti_uiaplus_loggers_stream_LoggerStreamer_TransportType_TCP
#define LoggerStreamer_TransportType_CUSTOM ti_uiaplus_loggers_stream_LoggerStreamer_TransportType_CUSTOM
#define LoggerStreamer_transportType ti_uiaplus_loggers_stream_LoggerStreamer_transportType
#define LoggerStreamer_initBuffer ti_uiaplus_loggers_stream_LoggerStreamer_initBuffer

#endif /* ti_uiaplus_loggers_stream_LoggerStreamer__localnames__done */
#endif

